@@ -11,6 +11,8 @@ Greenlet-local objects support the management of greenlet-local data.
 If you have data that you want to be local to a greenlet, simply create
 a greenlet-local object and use its attributes:
 
+  >>> import gevent
+  >>> from gevent.local import local
   >>> mydata = local()
   >>> mydata.number = 42
   >>> mydata.number
@@ -366,7 +368,7 @@ class local(object):
 
     def __cinit__(self, *args, **kw):
         if args or kw:
-            if type(self).__init__ == object.__init__:
+            if type(self).__init__ == object.__init__: # pylint:disable=comparison-with-callable
                 raise TypeError("Initialization arguments are not supported", args, kw)
         impl = _localimpl(args, kw, type(self), id(self))
         # pylint:disable=attribute-defined-outside-init
@@ -438,13 +440,23 @@ class local(object):
             return dct[name]
 
         if name in self._local_type_vars:
-            type_attr = getattr(self._local_type, name)
-
-            # It's not in the dict at all. Is it in the type?
+            # Not in the dictionary, but is found in the type. It could be
+            # a non-data descriptor still. Some descriptors, like @staticmethod,
+            # return objects (functions, in this case), that are *themselves*
+            # descriptors, which when invoked, again, would do the wrong thing.
+            # So we can't rely on getattr() on the type for them, we have to
+            # look through the MRO dicts ourself.
             if name not in self._local_type_get_descriptors:
-                # Not a descriptor, can't execute code
-                return type_attr
-            return type(type_attr).__get__(type_attr, self, self._local_type)
+                # Not a descriptor, can't execute code. So all we need is
+                # the return value of getattr() on our type.
+                return getattr(self._local_type, name)
+
+            for base in self._local_type.mro():
+                bd = base.__dict__
+                if name in bd:
+                    attr_on_type = bd[name]
+                    result = type(attr_on_type).__get__(attr_on_type, self, self._local_type)
+                    return result
 
         # It wasn't in the dict and it wasn't in the type.
         # So the next step is to invoke type(self)__getattr__, if it
@@ -569,25 +581,42 @@ def __new__(cls, *args, **kw):
     self.__cinit__(*args[1:], **kw)
     return self
 
-try:
+if local.__module__ == 'gevent.local':
     # PyPy2/3 and CPython handle adding a __new__ to the class
     # in different ways. In CPython and PyPy3, it must be wrapped with classmethod;
-    # in PyPy2, it must not. In either case, the args that get passed to
+    # in PyPy2 < 7.3.3, it must not. In either case, the args that get passed to
     # it are stil wrong.
-    local.__new__ = 'None'
-except TypeError: # pragma: no cover
-    # Must be compiled
-    pass
-else:
+    #
+    # Prior to Python 3.10, Cython-compiled classes were immutable and
+    # raised a TypeError on assignment to __new__, and we relied on that
+    # to detect the compiled version; but that breaks in
+    # 3.10 as classes are now mutable. (See
+    # https://github.com/cython/cython/issues/4326).
+    #
+    # That's OK; post https://github.com/gevent/gevent/issues/1480, the Cython-compiled
+    # module has a different name than the pure-Python version and we can check for that.
+    # It's not as direct, but it works.
+    # So here we're not compiled
     from gevent._compat import PYPY
     from gevent._compat import PY2
     if PYPY and PY2:
+        # The behaviour changed with no warning between PyPy2 7.3.2 and 7.3.3.
         local.__new__ = __new__
+        try:
+            local() # <= 7.3.2
+        except TypeError:
+            # >= 7.3.3
+            local.__new__ = classmethod(__new__)
     else:
         local.__new__ = classmethod(__new__)
 
     del PYPY
     del PY2
+else: # pragma: no cover
+    # Make sure we revisit in case of changes to the (accelerator) module names.
+    if local.__module__ != 'gevent._gevent_clocal':
+        raise AssertionError("Module names changed (local: %r; __name__: %r); revisit this code" % (
+            local.__module__, __name__) )
 
 _init()
 
